function example1
%example 1 : a diagonal problem
%shows how eigenvalues and eigenfunctions can be computed
disp('**************************************************')
disp('                EXAMPLE 1')
disp('  V(x)=[x^2,0;0,x^2] over [0,10]')
disp('**************************************************')
format long;
%the endpoints of the integration interval:
system.a=0;
system.b=10;
%boundary conditions:
system.A1=eye(2);
system.A2= zeros(2,2);
system.B1=eye(2);
system.B2= zeros(2,2);
%potential matrix function (function handle)
system.V=@potentialMatrix;
%computes eigenvalues with indices 0 to 10, with input tolerance 1e-8
disp('Computing Eigenvalues....')
[EigvData,meshData]=computeEigenvalues(system,0,10,1e-8);

%display results:
disp(['Number of intervals in the mesh: ' num2str(length(meshData.h))]); 
disp(sprintf('k \t  E_k \t\t\t\t ErrEst \t status'))
for i=1:length(EigvData.eigenvalues)
    disp(sprintf('%-3.0f\t %-16.12f\t %-2.0e\t\t %d', EigvData.indices(i),EigvData.eigenvalues(i),EigvData.errorEstimations(i),EigvData.status(i)));
end

%make plots of the eigenfunction corresponding to the eigenvalue 3 which
%has multiplicity 2
[x,Y,YP]=computeEigenfunction(system,meshData,3,2);
figure
plot(x,Y(1,:,1))
hold on
plot(x,Y(2,:,1),'r')
title('Eigenfunction corresponding to \lambda=3')
legend('y_1(x)','y_2(x)')
hold off
figure
plot(x,YP(1,:,1))
hold on
plot(x,YP(2,:,1),'r')
hold off
xlabel('x')
title('First derivative of eigenfunction corresponding to \lambda=3')
legend('y''_1(x)','y''_2(x)')
end

function r=potentialMatrix(x)
  %returns the potential matrix evaluated in x
  %x may be a vector of values: make sure that the entries are vectorized, e.g.
  % x.^2 instead of x^2.
  r=zeros(2,2,length(x));
  r(1,1,:)=x.^2;
  r(2,2,:)=x.^2;
end

