function example11
disp('**************************************************')
disp('     EXAMPLE 11: Woods-Saxon testproblem (n=8)')
disp('**************************************************')
format long;
%the endpoints of the integration interval:
system.a=0;
system.b=15;
%boundary conditions:
N=8;
system.A1=eye(N);
system.A2= zeros(N,N);
system.B1=eye(N);
system.B2= zeros(N,N);
%potential matrix function (function handle)
system.V=@potentialMatrix;
% 
x=0:0.1:15;
R=potentialMatrix(x);
plot(x,reshape(R(1,1,:),1,length(x)))
hold on
for k=2:N
     plot(x,reshape(R(k,k,:),1,length(x)))
end
hold off
title('Diagonal elements of the potential matrix')
xlabel('x')

%computes eigenvalues with indices 0 to 25, with input tolerance 1e-6
[EigvData,meshData]=computeEigenvalues(system,0,25,1e-6);
% 
% %display results:
disp(['Number of intervals in the mesh: ' num2str(length(meshData.h))]); 
disp(sprintf('k \t  E_k \t\t\t\t ErrEst \t status'))
for i=1:length(EigvData.eigenvalues)
    disp(sprintf('%-3.0f\t %-16.12f\t %-2.0e\t\t %d', EigvData.indices(i),EigvData.eigenvalues(i),EigvData.errorEstimations(i),EigvData.status(i)));
end

end

function R=potentialMatrix(x)
for i=1:length(x)
    R(:,:,i)=pot(x(i));
end
end

function res=pot(x)
M=[1   0    1/4  0     1/8     0      1/16 0;
0   7/8  0    3/16  0       3/32   0 3/64;
1/4 0    9/8  0     5/16    0      5/32 0 ;
0   3/16 0    31/32 0       15/64  0 15/128;
1/8 0    5/16 0     33/32   0       17/64  0 ;
0   3/32 0    15/64 0       127/128 0 63/256;
1/16 0    5/32 0     17/64   0       129/128 0;
0   3/64 0    15/128 0      63/256  0 511/512];
res=(-50*(1-5*(exp((x-7)/0.6))/(3*(1+(exp((x-7)/0.6)))))/(1+(exp((x-7)/0.6))))*M;
end
