function example2
disp('**************************************************')
disp('                EXAMPLE 2')
disp('  V(x)=[x^2,-1;-1,x^2] over [0,10]')
disp('**************************************************')
format long;
%the endpoints of the integration interval:
system.a=0;
system.b=10;
%boundary conditions:
system.A1=eye(2);
system.A2= zeros(2,2);
system.B1=eye(2);
system.B2= zeros(2,2);
%function handle to the function returning the potential matrix V
system.V=@potentialMatrix;
%eigenvalue computations:
disp('Computing Eigenvalues....')
[EigvData,meshData]=computeEigenvalues(system,0,20,1e-8);


disp(['Number of intervals in the mesh: ' num2str(length(meshData.h))]); %number of intervals in the partition
disp(sprintf('k \t  E_k \t\t\t\t ErrEst \t status'))
for i=1:length(EigvData.eigenvalues)
    disp(sprintf('%-3.0f\t %-16.12f\t %-2.0e\t\t %d', EigvData.indices(i),EigvData.eigenvalues(i),EigvData.errorEstimations(i),EigvData.status(i)));
end

%computes the eigenfunction corresponding to the eigenvalue = 40
[x,Y,YP]=computeEigenfunction(system,meshData,40,1);
figure
plot(x,Y(1,:,1))
hold on
plot(x,Y(2,:,1),'r')
hold off
xlabel('x')
title('Eigenfunction corresponding to \lambda=40')
legend('y_1(x)','y_2(x)')
figure
plot(x,YP(1,:,1))
hold on
plot(x,YP(2,:,1),'r')
hold off
xlabel('x')
title('First derivative of eigenfunction corresponding to \lambda=40')
legend('y''_1(x)','y''_2(x)')
end




function r=potentialMatrix(x)
  %returns the potential matrix evaluated in x
  %x may be a vector of values: make sure that the entries are vectorized, e.g.
  % x.^2 instead of x^2.
  r(1,1,:)=x.^2;
  r(2,2,:)=x.^2;
  r(1,2,:)=-1*ones(1,length(x));
  r(2,1,:)=-1*ones(1,length(x));
end

