function example3
%Marletta's testexample for the SL12F code.
disp('**************************************************')
disp('                EXAMPLE 3')
disp('  Marletta''s testexample for the SL12F code')
disp('**************************************************')
format long;
%the endpoints of the integration interval:
system.a=0.1;   
system.b=1; 
%parameters of the boundary conditions:
system.A1=eye(4);
system.A2= zeros(4,4);
system.B1=eye(4);
system.B2= zeros(4,4);
%function handle to the function returning the potential matrix
system.V=@potentialMatrix;



disp(' ');
disp('Calculating eigenvalues with indices between 0 and 10...');
t=cputime;
[EigvData,meshData]=computeEigenvalues(system,0,10,1e-8);

disp(['Number of intervals in the mesh: ' num2str(length(meshData.h))]); %number of intervals in the mesh
disp(['Time: ' num2str(cputime - t) ' sec']);
disp(sprintf('k \t  E_k \t\t\t\t ErrEst \t status'))
for i=1:length(EigvData.eigenvalues)
    disp(sprintf('%-3.0f\t %-16.12f\t %-2.0e\t\t %d', EigvData.indices(i),EigvData.eigenvalues(i),EigvData.errorEstimations(i),EigvData.status(i)));
end

%eigenfunction corresponding to the fifth eigenvalue:
[x,Y,YP]=computeEigenfunction(system,meshData,EigvData.eigenvalues(5),1);
figure
plot(x,Y(1,:,1))
hold on
plot(x,Y(2,:,1),'r')
plot(x,Y(3,:,1),'m')
plot(x,Y(4,:,1),'g')
hold off
xlabel('x')
title('Eigenfunction y_4(x)')

end




function r=potentialMatrix(x)
  %returns the potential matrix evaluated in x
  %x may be a vector of values: make sure that the entries are vectorized, e.g.
  % x.^2 instead of x^2.
  r=zeros(4,4,length(x));
  for i=1:4
    r(i,i,:)=(1/i)*cos(x)+(1./x.^i);
  end
  r(1,2,:)=(1/2)*cos(x);r(2,1,:)=(1/2)*cos(x);
  r(1,3,:)=(1/3)*cos(x);r(2,3,:)=(1/3)*cos(x);
  r(3,1,:)=(1/3)*cos(x);r(3,2,:)=(1/3)*cos(x);
  r(1,4,:)=(1/4)*cos(x);r(2,4,:)=(1/4)*cos(x);r(3,4,:)=(1/4)*cos(x);
  r(4,1,:)=(1/4)*cos(x);r(4,2,:)=(1/4)*cos(x);r(4,3,:)=(1/4)*cos(x);
end

