function example6
disp('**********************************************************************')
disp('                EXAMPLE 6')
disp('From: Bandyrskii, Gavrilyuk, Lazurchak and Makarov, Functional-');
disp('      discrete method (FD-method) for matrix Sturm-Liouville Problems,');
disp('      Computational Methods in Applied Mathematics, 5 (2005).')
disp('**********************************************************************')
format long;
system.a=0;   
system.b=1; 
system.A1=eye(2);
system.A2= zeros(2,2);
system.B1=eye(2);
system.B2= zeros(2,2);
system.V=@potentialMatrix;
disp(' ');
disp('Calculating eigenvalues with indices between 0 and 15...');
t=cputime;
[EigvData,meshData]=computeEigenvalues(system,0,15,1e-10);

disp(['Number of intervals in the mesh: ' num2str(length(meshData.h))]); %number of intervals in the partition
disp(['Time: ' num2str(cputime - t) ' sec']);
disp(sprintf('k \t  E_k \t\t\t\t ErrEst \t status'))
for i=1:length(EigvData.eigenvalues)
    disp(sprintf('%-3.0f\t %-16.12f\t %-2.0e\t\t %d', EigvData.indices(i),EigvData.eigenvalues(i),EigvData.errorEstimations(i),EigvData.status(i)));
end

[x,Y,YP]=computeEigenfunction(system,meshData.referenceMesh,EigvData.eigenvalues(10),1);
figure
plot(x,Y(1,:,1),':.')
hold on
plot(x,Y(2,:,1),'r:.')
hold off
xlabel('x')
title('Eigenfunction y_9(x)')
figure
plot(x,YP(1,:,1),':.')
hold on
plot(x,YP(2,:,1),'r:.')
hold off
xlabel('x')
title('First derivative of eigenfunction y_9(x)')

end




function r=potentialMatrix(x)
  %returns the potential matrix evaluated in x
  %x may be a vector of values: make sure that the entries are vectorized, e.g.
  % x.^2 instead of x^2.
  r(1,2,:)=-x/2;
  r(2,1,:)=-x/2;
  r(1,1,:)=3*x/2;
  r(2,2,:)=3*x/2;
end

