function example9
disp('**************************************************')
disp('                EXAMPLE 9: an exactly solvable system')
disp('  from: L.Gr.Ixaru, Physical Review A 77 (2008)')
disp('**************************************************')
format long;
%the endpoints of the integration interval:
system.a=0;
system.b=30;
%boundary conditions:
system.A1=eye(2);
system.A2= zeros(2,2);
system.B1=eye(2);
system.B2= zeros(2,2);
%potential matrix function (function handle)
system.V=@potentialMatrix;
% 
%computes eigenvalues with indices 0 to 10, with input tolerance 1e-9
[EigvData,meshData]=computeEigenvalues(system,0,10,1e-9);

%display results:
disp(['Number of intervals in the mesh: ' num2str(length(meshData.h))]); 
disp(sprintf('k \t  E_k \t\t\t\t ErrEst \t status'))
for i=1:length(EigvData.eigenvalues)
    disp(sprintf('%-3.0f\t %-16.12f\t %-2.0e\t\t %d', EigvData.indices(i),EigvData.eigenvalues(i),EigvData.errorEstimations(i),EigvData.status(i)));
end
 
%make plots of the eigenfunction corresponding to the first eigenvalue which
%has multiplicity 1
[x,Y,YP]=computeEigenfunction(system,meshData,EigvData.eigenvalues(1),1);
figure
plot(x,Y(1,:,1))
hold on
plot(x,Y(2,:,1),'r')
title('Eigenfunction')
legend('y_1(x)','y_2(x)')
hold off
figure
plot(x,YP(1,:,1))
hold on
plot(x,YP(2,:,1),'r')
hold off
xlabel('x')
title('First derivative of eigenfunction')
legend('y''_1(x)','y''_2(x)')


end

function r=potentialMatrix(x)
  %returns the potential matrix evaluated in x
  %x may be a vector of values: make sure that the entries are vectorized, e.g.
  % x.^2 instead of x^2.
  r=zeros(2,2,length(x));
  r(1,1,:)=-45./cosh(1*x).^2-(39/2)./cosh(1/2*x).^2;
  r(2,2,:)=-45./cosh(1*x).^2-(39/2)./cosh(1/2*x).^2;
  r(1,2,:)=-45./cosh(1*x).^2+(39/2)./cosh(1/2*x).^2;
  r(2,1,:)=-45./cosh(1*x).^2+(39/2)./cosh(1/2*x).^2;
end

